@@ -18,6 +18,8 @@
 #
 package Output;
 use strict;
+use open IO => ":utf8";
+
 BEGIN { @Namespace::ISA=qw(main); }
 
 # $objOutputter new()
@@ -119,11 +121,9 @@ sub output_wrap_vfunc_h($$$$$$)
     $cppVfuncDecl .= " const";
   }
 
-  $self->append("#ifdef GLIBMM_VFUNCS_ENABLED\n");
   $self->ifdef($ifdef);
   $self->append("  $cppVfuncDecl;\n");
   $self->endif($ifdef);
-  $self->append("#endif //GLIBMM_VFUNCS_ENABLED\n");
 
   #The default callback, which will call *_vfunc, which will then call the base default callback.
   #Declares the callback in the private *Class class and sets it in the class_init function.
@@ -140,51 +140,58 @@ sub output_wrap_vfunc_h($$$$$$)
 # _VFUNC_CC(signame,gtkname,rettype,crettype,`<cppargs>',`<cargs>')
 sub output_wrap_vfunc_cc($$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal, $ifdef) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objCFunc,
+      $custom_vfunc, $custom_vfunc_callback, $ifdef) = @_;
 
-  my $cname = $$objDefsSignal{name};
+  my $cname = $$objCFunc{name};
 
   # e.g. Gtk::Button::draw_indicator:
 
   #Use a different macro for Interfaces, to generate an extra convenience method.
 
-  my $refreturn = "";
-  $refreturn = "refreturn" if($$objCppfunc{rettype_needs_ref});
+  if ($custom_vfunc eq 0)
+  {
+    my $refreturn = "";
+    $refreturn = "refreturn" if($$objCppfunc{rettype_needs_ref});
 
-  my $str = sprintf("_VFUNC_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s)dnl\n",
-    $$objCppfunc{name},
-    $cname,
-    $$objCppfunc{rettype},
-    $$objDefsSignal{rettype},
-    $objCppfunc->args_types_and_names(),
-    convert_args_cpp_to_c($objCppfunc, $objDefsSignal, 0, $line_num), #$objCppfunc->args_names_only(),
-    $objCppfunc->get_is_const(),
-    $refreturn,
-    $ifdef);
+    my $str = sprintf("_VFUNC_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s)dnl\n",
+      $$objCppfunc{name},
+      $cname,
+      $$objCppfunc{rettype},
+      $$objCFunc{rettype},
+      $objCppfunc->args_types_and_names(),
+      convert_args_cpp_to_c($objCppfunc, $objCFunc, 0, $line_num), #$objCppfunc->args_names_only(),
+      $objCppfunc->get_is_const(),
+      $refreturn,
+      $ifdef);
 
-  $self->append($str);
+    $self->append($str);
+  }
 
   # e.g. Gtk::ButtonClass::draw_indicator():
 
-  my $refreturn_ctype = "";
-  $refreturn_ctype = "refreturn_ctype" if($$objDefsSignal{rettype_needs_ref});
+  if ($custom_vfunc_callback eq 0)
+  {
+    my $refreturn_ctype = "";
+    $refreturn_ctype = "refreturn_ctype" if($$objCFunc{rettype_needs_ref});
 
-  my $str = sprintf("_VFUNC_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s,%s,%s)dnl\n",
-    $$objCppfunc{name},
-    $cname,
-    $$objCppfunc{rettype},
-    $$objDefsSignal{rettype},
-    $objDefsSignal->args_types_and_names(),
-    $objDefsSignal->args_names_only(),
-    convert_args_c_to_cpp($objDefsSignal, $objCppfunc, $line_num),
-    ${$objDefsSignal->get_param_names()}[0],
-    $refreturn_ctype,
-    $ifdef);
+    my $str = sprintf("_VFUNC_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s,%s,%s)dnl\n",
+      $$objCppfunc{name},
+      $cname,
+      $$objCppfunc{rettype},
+      $$objCFunc{rettype},
+      $objCFunc->args_types_and_names(),
+      $objCFunc->args_names_only(),
+      convert_args_c_to_cpp($objCFunc, $objCppfunc, $line_num),
+      ${$objCFunc->get_param_names()}[0],
+      $refreturn_ctype,
+      $ifdef);
 
-  $self->append($str);
+    $self->append($str);
+  }
 }
 
-### Convert _WRAP to a virtual
+### Convert _WRAP to a signal
 # _SIGNAL_H(signame,rettype, ifdef, `<cppargs>')
 # _SIGNAL_PH(gtkname,crettype, ifdef, cargs and names)
 # void output_wrap_default_signal_handler_h($filename, $line_num, $objCppfunc, $objCDefsFunc, $ifdef. @args)
@@ -291,46 +298,9 @@ sub output_wrap_meth($$$$$$$)
   # Doxygen documentation before the method declaration:
   $self->output_wrap_meth_docs_only($filename, $line_num, $documentation);
 
- $self->ifdef($ifdef);
-
-  if($$objCDefsFunc{throw_any_errors})
-  {
-     $self->append("#ifdef GLIBMM_EXCEPTIONS_ENABLED\n");
-  }
+  $self->ifdef($ifdef);
 
   $self->append("  ${cppMethodDecl};");
-
-  if($$objCDefsFunc{throw_any_errors})
-  {
-     $self->append("\n#else\n");
-
-     # #Add an error argument, by searching for ) at the end and replacing it:
-     # my $declWithErrorArg = ${cppMethodDecl};
-     # $declWithErrorArg =~ s/\)$/, std::auto_ptr<Glib::Error>& error\)/g;
-
-     #Recreate the declaration, to remove the default values, which we can't have as well as an error parameter at the end: 
-     my $declWithErrorArg = $$objCppfunc{rettype} . " " . $$objCppfunc{name} . "(" . $objCppfunc->args_types_and_names() . ", std::auto_ptr<Glib::Error>& error)";
-
-     if($$objCppfunc{static})
-     {
-       $declWithErrorArg = "static " . $declWithErrorArg;
-     }
-
-     if($objCppfunc->get_is_const() eq 1)
-     {
-       if($$objCppfunc{static} ne 1) #It can't be const and static at the same time.
-       {
-         $declWithErrorArg = $declWithErrorArg . " const";
-       }
-     }
-
-     #remove any superfluous ,:
-     $declWithErrorArg =~ s/\(, /\(/g;
-
-     $self->append("  ${declWithErrorArg};");
-
-     $self->append("\n#endif //GLIBMM_EXCEPTIONS_ENABLED\n");
-  }
   
   $self->endif($ifdef);
 
@@ -618,7 +588,14 @@ sub output_wrap_property($$$$$$)
     my $name_underscored = $name;
     $name_underscored =~ tr/-/_/;
 
-    my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,%s)dnl\n",
+      # For the docs of the property (the final argument of the sprintf), the
+      # m4 quotes are changed, the new quotes are then used to quote the docs
+      # and then the quotes are changed back to the standard quotes.  This is
+      # done so that if there are commas in the docs, the contents after the
+      # docs are not lost (m4 thinks the contents after the comma is another
+      # agument to the macro).  Using the standard quotes leaves a trailing
+      # single quote for some undetermined reason.
+    my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,changequote([,])[%s]changequote(`,'))dnl\n",
       $name,
       $name_underscored,
       $cpp_type,
@@ -631,7 +608,15 @@ sub output_wrap_property($$$$$$)
     # If the property is not already read-only, and the property can be read, then add a second const accessor for a read-only propertyproxy:
     if( ($proxy_suffix ne "_ReadOnly") && ($objProperty->get_readable()) )
     {
-      my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,%s)dnl\n",
+
+      # For the docs of the property (the final argument of the sprintf), the
+      # m4 quotes are changed, the new quotes are then used to quote the docs
+      # and then the quotes are changed back to the standard quotes.  This is
+      # done so that if there are commas in the docs, the contents after the
+      # docs are not lost (m4 thinks the contents after the comma is another
+      # agument to the macro).  Using the standard quotes leaves a trailing
+      # single quote for some undetermined reason.
+      my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,changequote([,])[%s]changequote(`,'))dnl\n",
         $name,
         $name_underscored,
         $cpp_type,
@@ -640,8 +625,6 @@ sub output_wrap_property($$$$$$)
       );
       $self->append($str);
     }
-
-    
   }
 }
 
@@ -684,7 +667,7 @@ sub write_sections_to_files()
 
   open(INPUT, '<', "$$self{tmpdir}/gtkmmproc_$$.g2"); # $$ is the process ID.
 
-  # open tempory file for each section
+  # open temporary file for each section
   open(OUTPUT_H,  '>', "$fname_h.tmp");
   open(OUTPUT_PH, '>', "$fname_ph.tmp");
   open(OUTPUT_CC, '>', "$fname_cc.tmp");
